@@ -14,21 +14,36 @@ class Field(BaseField):
     Specialized field types can inherit from BaseField or this class.
     """
 
-    def _evaluateconditions(self, computed: t.Dict[str, t.Any]) -> t.Dict[str, t.Any]:
+    def _evaluateconditions(self, computed: t.Dict[str, t.Any], data: t.Any = None) -> t.Dict[str, t.Any]:
         """Evaluate all conditions given computed field values."""
-        if not self.dependencies:
+        if not self.dependencies and not self.selfdependent:
             raise ValueError(f"Conditional Field '{self.name}' has no dependencies")
         if not self.conditions:
             raise ValueError(f"Conditional Field '{self.name}' has no conditions")
 
         result = {}
 
+        # get fields own source value if selfdependent
+        fieldvalue = None
+        if (
+            self.selfdependent and
+            self.source is not None and
+            data is not None
+        ):
+            fieldvalue = self._getsourcevalue(data)
+
+
         depvals = [] # dependency values
+        if self.selfdependent:
+            depvals.append(fieldvalue)
+
+
         for dep in self.dependencies:
             if dep not in computed:
                 raise ValueError(f"Dependency '{dep}' not available for field '{self.name}'")
             depvals.append(computed[dep])
 
+
         for condition, evaluator in self.conditions.items():
             try:
                 result[condition] = evaluator(*depvals)
@@ -40,7 +55,7 @@ class Field(BaseField):
     def _extractwithoverrides(self, data: t.Any, overrides: t.Dict[str, t.Any]) -> t.Any:
         """Extract with temporary property overrides."""
         originals = {}
-        for prop in ['required', 'default', 'transform', 'choices', 'type', 'mapping']:
+        for prop in ['required', 'default', 'transform', 'choices', 'type', 'mapping', 'validator']:
             originals[prop] = getattr(self, prop)
 
         try:
@@ -78,8 +93,13 @@ class Field(BaseField):
         # handle conditional
         if self.conditional:
             if computed is None:
-                raise ValueError(f"Conditional field '{self.name}' requires computed (fields)")
-            evaluated = self._evaluateconditions(computed)
+                # only raise if field is required
+                if self.required:
+                    raise ValueError(f"Conditional field '{self.name}' requires computed (fields)")
+                else:
+                    # return default for non-required conditional fields
+                    return self.default
+            evaluated = self._evaluateconditions(computed, data)
 
             if 'value' in evaluated:
                 return evaluated['value']
@@ -149,13 +169,20 @@ class FallbackField(BaseField):
         Returns:
             Value from primary field, or fallback field if primary fails
         """
+        #print(f"DEBUG FallbackField.extract: data = {data}")
+        #print(f"DEBUG FallbackField.extract: computed = {computed}")
         try:
             value = self.primary.extract(data)
+            #print(f"DEBUG FallbackField.extract: value = {value}")
             # Consider None as a failure if primary is not required
             if value is None and not self.primary.required:
+                #print(f"DEBUG FallbackField.extract: falling back")
                 return self.fallback.extract(data)
             return value
-        except Exception:
+        except Exception as e:
+            if self.primary.validator is not None:
+                raise e
+            #print(f"DEBUG FallbackField.extract: exception = {e}")
             # Primary failed, try fallback
             return self.fallback.extract(data)
 
@@ -238,6 +265,9 @@ class CombinedField(BaseField):
                         result[f"field{len(result)}"] = value
 
             except Exception as e:
+                # if field has a validator, raise
+                if field.validator is not None:
+                    raise e
                 if field.required:
                     errors.append(f"Required field '{field.name}' failed: {e}")
                 # Non-required fields that fail are just skipped
@@ -499,6 +529,8 @@ class AccumulatedField(BaseField):
                 if value is not None:
                     values.append(value)
             except Exception as e:
+                if field.validator is not None:
+                    raise e
                 if field.required:
                     raise ValueError(f"Required field '{field.name}' failed in accumulation: {e}")
                 # Skip non-required fields that fail
@@ -686,7 +718,9 @@ class SourceField(Field):
             if (result == self.default):
                 return self._extractwithfallbacks(data)
             return result
-        except (ValueError, KeyError, AttributeError):
+        except Exception as e:
+            if self.validator is not None:
+                raise e
             # Try fallback sources
             return self._extractwithfallbacks(data)
 
@@ -700,6 +734,7 @@ class SourceField(Field):
         Returns:
             First successful extraction or default value
         """
+        #print(f"DEBUG SourceField._extractwithfallbacks: data = {data}")
         originalsource = self.source
 
         for fallbacksource in self.fallbacks:
@@ -707,8 +742,13 @@ class SourceField(Field):
                 # Temporarily set fallback as source
                 self.source = fallbacksource
                 result = super().extract(data)
+                #print(f"DEBUG SourceField._extractwithfallbacks: result = {result}")
                 return result
-            except Exception:
+            except Exception as e:
+                if self.validator is not None:
+                    raise e
+                #print(f"DEBUG SourceField._extractwithfallbacks: exception = {e}")
+
                 continue
             finally:
                 # Restore original source
@@ -719,6 +759,7 @@ class SourceField(Field):
             sources = [originalsource] + self.fallbacks
             raise ValueError(f"Required field '{self.name}' not found in any sources: {sources}")
 
+        #print(f"DEBUG SourceField._extractwithfallbacks: returning default = {self.default}")
         return self.default
 
     def addfallback(self, source_path: str) -> 'SourceField':
